/*****************************************************************************
 *
 *  BoolListParameter.cc                        XMM Science Analysis Subsystem
 *
 *  Boolean-list task parameter
 *
 *  (c) 1998  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: BoolListParameter.cc,v 1.18 2001/03/28 19:20:34 jbakker Exp $
 *
 *****************************************************************************/

#include <strstream>             // C++ headers
#include <iostream>
#include <string>
#include <vector>

#include "BoolListParameter.h"     // Application headers
#include "StringToken.h"
#include <errstr.h>
#include <StringFns.h>
#include "freezestream.h"


//================================================================
//  Class BoolListParameter
//================================================================

/*
 *  All instances of vector<bool> have been replaced by vector<int> to
 *  work-around a problem with the egcs compiler. Consequently, setValue()
 *  cannot be used at present (unless a vector<int> argument is used).
 */


/*** Format the parameter value as a string in quotes ***/

string BoolListParameter::asString() const
{
    ostrstream ostr;
    vector<int>::const_iterator it = _value.begin();
    ostr << '\'';
    if(it != _value.end())
        ostr << bool(*it++);
    while(it != _value.end())
        ostr << ' ' << bool(*it++);
    ostr << '\'' << ends;

    return freezestream(ostr);
}


/*** Format the default value as a string in quotes ***/

string BoolListParameter::defaultAsString() const
{
    ostrstream ostr;
    vector<int>::const_iterator it = _default.begin();
    ostr << '\'';
    if(it != _default.end())
        ostr << bool(*it++);
    while(it != _default.end())
        ostr << ' ' << bool(*it++);
    ostr << '\'' << ends;

    return freezestream(ostr);
}


/*
 *  Parse string representation and assign to 'value' if it is valid.
 *  The validation is not strictly necessary, since bool must be valid.
 */

bool BoolListParameter::parse(const string& str, vector<int>& value)
{
    list<string> stringTokens = StringFns::splitString(string(str)," \t");

    bool ok = true;
    list<string>::const_iterator it = stringTokens.begin();
    while(ok && it != stringTokens.end())
    {
        string s = *it++;
	StringToken t(s);
	bool i = t.asBool();
	if(t.isValid())
	    value.push_back(i);
	else {
	  errstr << "Parameter '" << _name << "'" << Sas::error("ParamWrongType");
	  ok = false;
	}
    }
    return ok;
}


/*** Revert to default value ***/

void BoolListParameter::revertToDefault()
{
    _value = _default;
    notifyView();
}


/*** Test whether the parameter is valid ***/

bool BoolListParameter::validate() const
{
     return isValid(_value);
}


/*** Test whether value is allowable for the parameter ***/

bool BoolListParameter::isValid(const vector<int>&) const
{
    return true;
}


/*** Set the parameter to the specified value ***/

bool BoolListParameter::setValue(const vector<int>& value)
{
    bool valid = isValid(value);
    if(valid)
    {
        _value = value;
	setDefined();
	notifyView();
    }
    return valid;
}


/*** Set the default to the specified value ***/

bool BoolListParameter::setDefault(const vector<int>& value)
{    
    bool valid = isValid(value);
    if(valid)
        _default = value;
    return valid;
}


/*** Set the value, specified as a string ***/

bool BoolListParameter::setNewValue(const string& str)
{
    vector<int> values;
    bool ok =  parse(str, values) && setValue(values);

    if(!ok)
        errstr << "Parameter '" << _name << "' cannot be assigned new value '"
	       << str << "'" << Sas::error("ParamNotAssigned");

    notifyView();
    return true;
}


/***  Set the default, specified as a string ***/

bool BoolListParameter::setNewDefault(const string& str)
{
    vector<int> values;
    bool ok = parse(str, values) && setDefault(values);

    if(!ok)
        errstr << "Parameter '" << _name << "' cannot be assigned default value "
	       << str << "'" << Sas::error("ParamNotAssigned");
    return true;
}


/*** Return the length of the list ***/

int BoolListParameter::size() const
{
    return _value.size();
}
