/*****************************************************************************
 *
 *  IntParameter.cc                             XMM Science Analysis Subsystem
 *
 *  Integer task parameter
 *
 *  (c) 1998  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: IntParameter.cc,v 1.11 2001/03/28 19:20:35 jbakker Exp $
 *
 *****************************************************************************/

#include <strstream>             // C++ headers
#include <iostream>

#include "IntParameter.h"          // Application headers
#include "StringToken.h"
#include <errstr.h>
#include "freezestream.h"


//================================================================
//  Class IntParameter
//================================================================

/*** Format the parameter value as a string ***/

string IntParameter::asString() const
{
    ostrstream ostr;
    ostr << _value << ends;
    return freezestream(ostr);
}


/*** Format the default value as a string ***/

string IntParameter::defaultAsString() const
{
    ostrstream ostr;
    ostr << _default << ends;
    return freezestream(ostr);
}


/***  Parse string representation and assign to 'value' if it is valid ***/

bool IntParameter::parse(const string& str, int& value)
{
    StringToken t(str);
    int n = t.asInt();
    if(t.isValid())
        value = n;
    else
      errstr << "Parameter '" << _name << "'" << Sas::error("ParamWrongType");

    return true;
}


/*** Write a text representation of the constraint ***/

void IntParameter::writeConstraint(ostream& os) const
{
    os << '(';
    if(_min.isDefined())
        os << _min << "<=";
    os << _name;
    if(_max.isDefined())
        os << "<=" << _max;
    os << ")";
}


/*** Revert to default value ***/

void IntParameter::revertToDefault()
{
    _value = _default;
    notifyView();
}


/*** Test whether the parameter is valid ***/

bool IntParameter::validate() const
{
    return isValid(_value);
}


/*** Test whether value is allowable for the parameter ***/

bool IntParameter::isValid(int n) const
{
    return (!_min.isDefined() || (n >= _min))
	&& (!_max.isDefined() || (n <= _max));
}


/*** Set the parameter to the specified value ***/

bool IntParameter::setValue(int value)
{
    bool valid = isValid(value);
    if(valid)
    {
        _value = value;
	setDefined();
	notifyView();
    }
    return valid;
}


/*** Set the default to the specified value ***/

bool IntParameter::setDefault(int value)
{
    bool valid = isValid(value);
    if(valid)
        _default = value;
    return valid;
}


/*** Set the value, specified as a string ***/

bool IntParameter::setNewValue(const string& str)
{
    int value;
    bool ok = parse(str, value) && setValue(value);

    if(!ok)
        reportRangeError(str);

    notifyView();
    return true;
}


/***  Set the default, specified as a string ***/

bool IntParameter::setNewDefault(const string& str)
{
    int value;
    bool ok = parse(str, value) && setDefault(value);

    if(!ok)
        reportRangeError(str);

    return true;
}


/*** Report that value is out of range ***/

void IntParameter::reportRangeError(const string& valueString)
{
  errstr << "Parameter '" << _name << "';" << endl
	 << "value '" << valueString << "' is outside range ";
  writeConstraint(errstr);
  errstr << Sas::error("ParamNotInRange");
}
