/*****************************************************************************
 *
 *  ParameterInfo.cc                            XMM Science Analysis Subsystem
 *
 *  Information about parameters of a task
 *
 *  (c) 1998  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: ParameterInfo.cc,v 1.22 2001/11/19 11:50:29 jbakker Exp $
 *
 *****************************************************************************/

#include "ParameterInfo.h"     // Application headers
#include "Parameter.h"
#include <errstr.h>

#include <iostream>          // C++ headers
#include <strstream>
#include "freezestream.h"


//================================================================
//  Class ParameterInfo
//================================================================

/*** Constructor ***/

ParameterInfo::ParameterInfo()
{
}


/***  Copy constructor (deep copy, including parameters) ***/

ParameterInfo::ParameterInfo(const ParameterInfo& info)
{
    copy(info);
} 


/*** Destructor ***/

ParameterInfo::~ParameterInfo()
{
  for (ParamMap::const_iterator it = _map.begin();it != _map.end();it++)
    delete (*it).second;
}


/*** Assignment operator ***/

ParameterInfo& ParameterInfo::operator=(const ParameterInfo& rhs)
{
    if (this != &rhs) copy(rhs);      
    return *this;
}


/*** Copy [ PRIVATE ] ***/

void ParameterInfo::copy(const ParameterInfo& info)
{
    _taskName = info._taskName;
    _nameList = info._nameList;

    ParameterInfo::ParamMap::const_iterator it2 = info._map.begin();
    while(it2 != info._map.end())
    {
        Parameter* p = (*it2++).second->clone();
	p->deleteView();
	string name = p->name();
        _map[name] = p;
    }
}


/*** Set the name of the task ***/

void ParameterInfo::setTaskName(const string& name)
{
    _taskName = name;
}


/*** Write the parameter information to a stream ***/

ostream& operator<<(ostream& os, const ParameterInfo& p)
{
  for (list<string>::const_iterator lit = p._nameList.begin();
       lit != p._nameList.end(); lit++) {
    
    ParameterInfo::ParamMap::const_iterator mit=p._map.find(*lit);
    if (mit == p._map.end()) errstr << "Could not fetch parameter '" << (*lit)
				  << "' from memory!" 
				  << Sas::error("ParamCorrupted");
    Parameter* p=(*mit).second;
    os << p->name() << " = " << (*p) << endl;
  }
  return os;
}


/*** Write the parameter information, including defaults, etc ***/

void ParameterInfo::writeHelpToStream(ostream& os) const
{
  // produce list as read from stream.
  for (list<string>::const_iterator lit = _nameList.begin();
       lit != _nameList.end(); lit++) {

    ParameterInfo::ParamMap::const_iterator mit=_map.find(*lit);
    if (mit == _map.end()) errstr << "Could not fetch parameter '" << (*lit)
				  << "' from memory!" 
				  << Sas::error("ParamCorrupted");
    Parameter* p=(*mit).second;
    p->writeHelpToStream(os);
  }
}


/*** Format the ParameterInfo as a command line ***/

string ParameterInfo::asCommandLine() const
{
  ostrstream os;
  os << _taskName;

  for (list<string>::const_iterator lit = _nameList.begin();
       lit != _nameList.end(); lit++) {

    ParameterInfo::ParamMap::const_iterator mit=_map.find(*lit);
    if (mit == _map.end()) errstr << "Could not fetch parameter '" << (*lit)
				  << "' from memory!" 
				  << Sas::error("ParamCorrupted");
    
    Parameter* p=(*mit).second;
    os << " " << p->name() << "=" << (*p);
  }
  //---
  os << ends;
  return freezestream(os);
}


/*** Format the ParameterInfo as an argument vector ***/

vector<string> ParameterInfo::asArgumentList() const
{
  vector<string> arglist;
  arglist.push_back(_taskName);

  for (list<string>::const_iterator lit = _nameList.begin();
       lit != _nameList.end(); lit++) {
    
    
    ParameterInfo::ParamMap::const_iterator mit=_map.find(*lit);
    if (mit == _map.end()) errstr << "Could not fetch parameter '" << (*lit)
				  << "' from memory!" 
				  << Sas::error("ParamCorrupted");
    Parameter* p=(*mit).second;
    string value = p->asString();
    if(value[0] == '\'') value = value.substr(1,value.size()-2);
    arglist.push_back(p->name() + "=" + value);
  }

  return arglist;
}


/*** Parse command line and set values of specified parameters ***/

void ParameterInfo::parseCommandLine(int& argc, char** argv)
{
    char* clmode = getenv("SAS_CLMODE");
    if(clmode)
    {
        string s = clmode;
	if(s == "PCS")
	    parsePCSCommandLine(argc, argv);
	else if(s == "SAS")
	    parseSASCommandLine(argc, argv);
	else
	    errstr << "Environment variable SAS_CLMODE = " 
		   << clmode << " not valid" << Sas::error("ParamEnvVariable");
    } else 
        parseSASCommandLine(argc, argv);
}


/*** Parse normal SAS command-line format ***/

void ParameterInfo::parseSASCommandLine(int& argc, char** argv)
{
    for(int i=0; i<argc; i++)
    {
        string arg = argv[i];

        // Ignore everything after "--"
	if(arg == "--") break;

        // Only process it when it is a=b style
        unsigned n = arg.find("=");
        if (n == string::npos) continue;

        string name;

        // Accept "foo" or "--foo", but ignore "-foo"
	if (arg.substr(0,2) == "--")
            name = arg.substr(2,n-2);
        else if (arg.substr(0,2) == "-")
            continue;
        else
            name = arg.substr(0,n);

        string value = arg.substr(n+1,string::npos);

        // Ignore parameters which are not in the parameter file
        if(_map.count(name) <= 0) continue;
	
        // Assign value to the parameter
	Parameter *p = _map[name];
        if(p && !p->setNewValue(value))
	    errstr << "Could not assign value " << value << " to " << name << endl;
	
        // Indicate the options used by setting them to 0
        argv[i] = 0;
    }

    // Remove all arguments that have been set to 0
    for (int i = 0; i < argc; ) {
        if (argv[i]) 
	    ++i;
        else {
	    for (int j = i; j < argc-1; ++j)
	        argv[j] = argv[j+1];
	    --argc;
	}
    }
}


/*** Parse special PCS command-line format ***/

void ParameterInfo::parsePCSCommandLine(int& argc, char** argv)
{
    typedef map<string,string,less<string> > OptionMap;
    OptionMap options;
    string mode = "odf";           // Default PCS keyword
  
    for (int i = 1; i < argc; ++i)
    {
        string arg = argv[i];

	if (arg.substr(0,2) == "--") {
	    // Ignore "--" options

	} else if (arg[0] == '-') {

	    // Switch mode on PCS keywords
	    if (arg == "-coeff" || arg == "-ccf" || arg == "-sp")
	    {
	        mode = arg.substr(1,string::npos);
		argv[i] = 0;
	    }
	} else {

	    // Append argument to value for current PCS keyword
	    if (!options[mode].empty()) 
	        options[mode] += " ";
	    options[mode] += argv[i];
	    argv[i] = 0;
	}
    }

    // Add the parameters
    for (OptionMap::const_iterator ix = options.begin(); ix != options.end(); ++ix)
    {
        string name  = (*ix).first;
	string value = (*ix).second;

        if(_map.count(name) > 0)    // Parameter exists
	{
	    Parameter *p = _map[name];
	    if(p && !p->setNewValue(value))
	        errstr << "Could not assign value " << value << 
		  " to " << name << endl;
	}
    }

    // Remove all argument that have been set to 0
    for (int i = 0; i < argc; )
    {
        if (argv[i]) ++i;
	else {
	    for (int j = i; j < argc-1; ++j)
	        argv[j] = argv[j+1];
	    --argc;
	}
    }
}


/*** Add a new parameter ***/

void ParameterInfo::addParameter(Parameter* p)
{
  const string& name=p->name();
  _map[name]=p;
  
  list<string>::const_iterator it = _nameList.begin();
  while (it != _nameList.end() && (*it) != name) it++;
  if (it == _nameList.end())  _nameList.push_back(name);
}


/*** Return the list of parameter names (in order) ***/

list<string> ParameterInfo::names() const
{
    return _nameList;
}
 

/*** Return the parameter "name" ***/

Parameter* ParameterInfo::operator[](const string& name)
{
    return parameter(name, true);
}


/*** Return the parameter "name" ***/

Parameter* ParameterInfo::parameter(const string& name, bool alertError)
{
    if(_map.count(name) > 0)
        return _map[name];
    else {
        if(alertError)
  	    errstr << "Parameter '" << name << "' not found in parameter space of '"
		   << _taskName << "'"
		   << Sas::error("ParamNotFound");
        return 0;
    }
}


/*** Revert each parameter to its default value ***/

void ParameterInfo::revertToDefault()
{
    list<string>::iterator iter = _nameList.begin();
    while(iter != _nameList.end())
    {
        Parameter* p = parameter(*iter++,true);
        if(p)
            p->revertToDefault();
    }
}


/** Check that all mandatory parameters have been set ***/

void ParameterInfo::checkMandatoryParameters()
{
    list<string>::const_iterator iter = _nameList.begin();
    string missing;
    while(iter != _nameList.end())
    {
        Parameter* p = parameter(*iter++);
        if(p && p->isMandatory() && !p->isSet())
	{
	  if (!missing.empty()) missing += ", ";
	  missing += p->name();
	}
    }
    if (!missing.empty())
    {
        errstr << "Please specify mandatory parameters:'"
	       << missing 
	       << "' belonging to parameter space of '"
	       << _taskName << "'"
	       << Sas::error("ParamMandatory");
    }
}
