/*****************************************************************************
 *
 *  RealListParameter.cc                        XMM Science Analysis Subsystem
 *
 *  Real list task parameter
 *
 *  (c) 1998  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: RealListParameter.cc,v 1.19 2001/03/28 19:20:36 jbakker Exp $
 *
 *****************************************************************************/

#include <strstream>             // C++ headers
#include <iostream>
#include <string>
#include <vector>

#include "RealListParameter.h"     // Application headers
#include "StringToken.h"
#include <errstr.h>
#include <StringFns.h>
#include "freezestream.h"


//================================================================
//  Class RealListParameter
//================================================================

/*** Format the parameter value as a string in quotes ***/

string RealListParameter::asString() const
{
    ostrstream ostr;
    vector<double>::const_iterator it = _value.begin();
    ostr << '\'';
    if(it != _value.end())
        ostr << doubleAsString(*it++);
    while(it != _value.end())
        ostr << ' ' << doubleAsString(*it++);
    ostr << '\'' << ends;

    return freezestream(ostr);
}


/*** Format the parameter value as a string in quotes ***/

string RealListParameter::defaultAsString() const
{
    ostrstream ostr;
    vector<double>::const_iterator it = _default.begin();
    ostr << '\'';
    if(it != _default.end())
        ostr << doubleAsString(*it++);
    while(it != _default.end())
        ostr << ' ' << doubleAsString(*it++);
    ostr << '\'' << ends;

    return freezestream(ostr);
}


/*** Format a single list element as a string ***/

string RealListParameter::doubleAsString(double value) const
{
    ostrstream ostr;
    ostr.precision(12);
    ostr << value << ends;
    return freezestream(ostr);
}


/***  Parse string representation and assign to 'value' if it is valid ***/

bool RealListParameter::parse(const string& str, vector<double>& value)
{
// TEMPORARY WORKAROUND TO ALLOW TIMES TO BE QUOTED

    list<string> stringTokens = StringFns::splitStringList(string(str));
//    list<string> stringTokens = StringFns::splitString(string(str)," \t");

    bool ok = true;
    bool first = true;
    list<string>::const_iterator it = stringTokens.begin();
    while(ok && it != stringTokens.end())
    {
        string token = *it++;
	double item = 0.0;
	ok &= parseDouble(token, item, first);
	first = false;
	if(ok)
	    value.push_back(item);
	else {
	  errstr << "Parameter '" << _name << "'" << Sas::error("ParamWrongType");
	  ok = false;
	}
    }
    return ok;
}


/*** Parse a single list element ***/

bool RealListParameter::parseDouble(const string& s, double& result, bool)
{
    StringToken token(s);
    result = token.asDouble();

    return token.isValid();
}


/*** Write a text representation of the constraint ***/

void RealListParameter::writeConstraint(ostream& os) const
{
    os << '(';
    if(_min.isDefined())
        os << _min << "<=";
    os << _name;
    if(_max.isDefined())
        os << "<=" << _max;
    os << ")";
}


/*** Revert to default value ***/

void RealListParameter::revertToDefault()
{
    _value = _default;
    notifyView();
}


/*** Test whether the parameter is valid ***/

bool RealListParameter::validate() const
{
     return isValid(_value);
}


/*** Test whether value is allowable for the parameter ***/

bool RealListParameter::isValid(const vector<double>& value) const
{
    bool valid = true;
    vector<double>::const_iterator it = value.begin();
    while(it != value.end())
    {
        double v = *it++;
	if(_min.isDefined())
	    valid &= (v >= _min);
	if(_max.isDefined())
	    valid &= (v <= _max);
    }
    return valid;
}


/*** Set the parameter to the specified value ***/

bool RealListParameter::setValue(const vector<double>& value)
{
    bool valid = isValid(value);
    if(valid)
    {
        _value = value;
	setDefined();
	notifyView();
    }
    return valid;
}


/*** Set the default to the specified value ***/

bool RealListParameter::setDefault(const vector<double>& value)
{
    bool valid = isValid(value);
    if(valid)
        _default = value;
    return valid;
}


/*** Set the value, specified as a string ***/

bool RealListParameter::setNewValue(const string& str)
{
    vector<double> values;
    bool ok =  parse(str, values) && setValue(values);

    if(!ok)
       reportRangeError(str);

    notifyView();
    return true;
}


/***  Set the default, specified as a string ***/

bool RealListParameter::setNewDefault(const string& str)
{
    vector<double> values;
    bool ok = parse(str, values) && setDefault(values);

    if(!ok)
        reportRangeError(str);

    return true;
}


int RealListParameter::size() const
{
    return _value.size();
}


/*** Report that value is out of range ***/

void RealListParameter::reportRangeError(const string& valueString)
{
  errstr << "Parameter '" << _name << "';" << endl
	 << "value '" << valueString << "' is outside range ";
  writeConstraint(errstr);
  errstr << Sas::error("ParamNotInRange");
}
